@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2015 The python-bitcoinlib developers
+# Copyright (C) 2012-2017 The python-bitcoinlib developers
 #
 # This file is part of python-bitcoinlib.
 #
@@ -9,21 +9,22 @@
 # propagated, or distributed except according to the terms contained in the
 # LICENSE file.
 
-from __future__ import absolute_import, division, print_function
 
 import binascii
 import struct
-import sys
 import time
 
-from .script import CScript
+from . import script
+from .script import CScript, CScriptWitness, OP_RETURN
 
 from .serialize import *
 
 # Core definitions
 COIN = 100000000
 MAX_BLOCK_SIZE = 1000000
+MAX_BLOCK_WEIGHT = 4000000
 MAX_BLOCK_SIGOPS = MAX_BLOCK_SIZE/50
+WITNESS_COINBASE_SCRIPTPUBKEY_MAGIC = bytes([OP_RETURN, 0x24, 0xaa, 0x21, 0xa9, 0xed])
 
 def MoneyRange(nValue, params=None):
     global coreparams
@@ -32,30 +33,14 @@ def MoneyRange(nValue, params=None):
 
     return 0 <= nValue <= params.MAX_MONEY
 
-def _py2_x(h):
-    """Convert a hex string to bytes"""
-    return binascii.unhexlify(h)
-
 def x(h):
     """Convert a hex string to bytes"""
     return binascii.unhexlify(h.encode('utf8'))
 
-def _py2_b2x(b):
-    """Convert bytes to a hex string"""
-    return binascii.hexlify(b)
-
 def b2x(b):
     """Convert bytes to a hex string"""
     return binascii.hexlify(b).decode('utf8')
 
-def _py2_lx(h):
-    """Convert a little-endian hex string to bytes
-
-    Lets you write uint256's and uint160's the way the Satoshi codebase shows
-    them.
-    """
-    return binascii.unhexlify(h)[::-1]
-
 def lx(h):
     """Convert a little-endian hex string to bytes
 
@@ -64,14 +49,6 @@ def lx(h):
     """
     return binascii.unhexlify(h.encode('utf8'))[::-1]
 
-def _py2_b2lx(b):
-    """Convert bytes to a little-endian hex string
-
-    Lets you show uint256's and uint160's the way the Satoshi codebase shows
-    them.
-    """
-    return binascii.hexlify(b[::-1])
-
 def b2lx(b):
     """Convert bytes to a little-endian hex string
 
@@ -80,18 +57,6 @@ def b2lx(b):
     """
     return binascii.hexlify(b[::-1]).decode('utf8')
 
-if not (sys.version > '3'):
-    x = _py2_x
-    b2x = _py2_b2x
-    lx = _py2_lx
-    b2lx = _py2_b2lx
-
-del _py2_x
-del _py2_b2x
-del _py2_lx
-del _py2_b2lx
-
-
 def str_money_value(value):
     """Convert an integer money value to a fixed point string"""
     r = '%i.%08i' % (value // COIN, value % COIN)
@@ -304,11 +269,85 @@ class CMutableTxOut(CTxOut):
         """Create a fullly mutable copy of an existing TxOut"""
         return cls(txout.nValue, txout.scriptPubKey)
 
+
+class CTxInWitness(ImmutableSerializable):
+    """Witness data for a single transaction input"""
+    __slots__ = ['scriptWitness']
+
+    def __init__(self, scriptWitness=CScriptWitness()):
+        object.__setattr__(self, 'scriptWitness', scriptWitness)
+
+    def is_null(self):
+        return self.scriptWitness.is_null()
+
+    @classmethod
+    def stream_deserialize(cls, f):
+        scriptWitness = CScriptWitness.stream_deserialize(f)
+        return cls(scriptWitness)
+
+    def stream_serialize(self, f):
+        self.scriptWitness.stream_serialize(f)
+
+    def __repr__(self):
+        return "CTxInWitness(%s)" % (repr(self.scriptWitness))
+
+    @classmethod
+    def from_txinwitness(cls, txinwitness):
+        """Create an immutable copy of an existing TxInWitness
+
+        If txin is already immutable (txin.__class__ is CTxIn) it is returned
+        directly.
+        """
+        if txinwitness.__class__ is CTxInWitness:
+            return txinwitness
+
+        else:
+            return cls(txinwitness.scriptWitness)
+
+class CTxWitness(ImmutableSerializable):
+    """Witness data for all inputs to a transaction"""
+    __slots__ = ['vtxinwit']
+
+    def __init__(self, vtxinwit=()):
+        object.__setattr__(self, 'vtxinwit', vtxinwit)
+
+    def is_null(self):
+        for n in range(len(self.vtxinwit)):
+            if not self.vtxinwit[n].is_null(): return False
+        return True
+
+    # FIXME this cannot be a @classmethod like the others because we need to
+    # know how many items to deserialize, which comes from len(vin)
+    def stream_deserialize(self, f):
+        vtxinwit = tuple(CTxInWitness.stream_deserialize(f) for dummy in
+                range(len(self.vtxinwit)))
+        return CTxWitness(vtxinwit)
+
+    def stream_serialize(self, f):
+        for i in range(len(self.vtxinwit)):
+            self.vtxinwit[i].stream_serialize(f)
+
+    def __repr__(self):
+        return "CTxWitness(%s)" % (','.join(repr(w) for w in self.vtxinwit))
+
+    @classmethod
+    def from_txwitness(cls, txwitness):
+        """Create an immutable copy of an existing TxWitness
+
+        If txwitness is already immutable (txwitness.__class__ is CTxWitness) it is returned
+        directly.
+        """
+        if txwitness.__class__ is CTxWitness:
+            return txwitness
+        else:
+            return cls(txwitness.vtxinwit)
+
+
 class CTransaction(ImmutableSerializable):
     """A transaction"""
-    __slots__ = ['nVersion', 'vin', 'vout', 'nLockTime']
+    __slots__ = ['nVersion', 'vin', 'vout', 'nLockTime', 'wit']
 
-    def __init__(self, vin=(), vout=(), nLockTime=0, nVersion=1):
+    def __init__(self, vin=(), vout=(), nLockTime=0, nVersion=1, witness=CTxWitness()):
         """Create a new transaction
 
         vin and vout are iterables of transaction inputs and outputs
@@ -318,30 +357,68 @@ class CTransaction(ImmutableSerializable):
         if not (0 <= nLockTime <= 0xffffffff):
             raise ValueError('CTransaction: nLockTime must be in range 0x0 to 0xffffffff; got %x' % nLockTime)
         object.__setattr__(self, 'nLockTime', nLockTime)
-
         object.__setattr__(self, 'nVersion', nVersion)
         object.__setattr__(self, 'vin', tuple(CTxIn.from_txin(txin) for txin in vin))
         object.__setattr__(self, 'vout', tuple(CTxOut.from_txout(txout) for txout in vout))
+        object.__setattr__(self, 'wit', CTxWitness.from_txwitness(witness))
 
     @classmethod
     def stream_deserialize(cls, f):
-        nVersion = struct.unpack(b"<i", ser_read(f,4))[0]
-        vin = VectorSerializer.stream_deserialize(CTxIn, f)
-        vout = VectorSerializer.stream_deserialize(CTxOut, f)
-        nLockTime = struct.unpack(b"<I", ser_read(f,4))[0]
-        return cls(vin, vout, nLockTime, nVersion)
+        """Deserialize transaction
 
-    def stream_serialize(self, f):
+        This implementation corresponds to Bitcoin's SerializeTransaction() and
+        consensus behavior. Note that Bitcoin's DecodeHexTx() also has the
+        option to attempt deserializing as a non-witness transaction first,
+        falling back to the consensus behavior if it fails. The difference lies
+        in transactions which have zero inputs: they are invalid but may be
+        (de)serialized anyway for the purpose of signing them and adding
+        inputs. If the behavior of DecodeHexTx() is needed it could be added,
+        but not here.
+        """
+        # FIXME can't assume f is seekable
+        nVersion = struct.unpack(b"<i", ser_read(f,4))[0]
+        pos = f.tell()
+        markerbyte = struct.unpack(b'B', ser_read(f, 1))[0]
+        flagbyte = struct.unpack(b'B', ser_read(f, 1))[0]
+        if markerbyte == 0 and flagbyte == 1:
+            vin = VectorSerializer.stream_deserialize(CTxIn, f)
+            vout = VectorSerializer.stream_deserialize(CTxOut, f)
+            wit = CTxWitness(tuple(0 for dummy in range(len(vin))))
+            wit = wit.stream_deserialize(f)
+            nLockTime = struct.unpack(b"<I", ser_read(f,4))[0]
+            return cls(vin, vout, nLockTime, nVersion, wit)
+        else:
+            f.seek(pos) # put marker byte back, since we don't have peek
+            vin = VectorSerializer.stream_deserialize(CTxIn, f)
+            vout = VectorSerializer.stream_deserialize(CTxOut, f)
+            nLockTime = struct.unpack(b"<I", ser_read(f,4))[0]
+            return cls(vin, vout, nLockTime, nVersion)
+
+
+    def stream_serialize(self, f, include_witness=True):
         f.write(struct.pack(b"<i", self.nVersion))
-        VectorSerializer.stream_serialize(CTxIn, self.vin, f)
-        VectorSerializer.stream_serialize(CTxOut, self.vout, f)
+        if include_witness and not self.wit.is_null():
+            assert(len(self.wit.vtxinwit) <= len(self.vin))
+            f.write(b'\x00') # Marker
+            f.write(b'\x01') # Flag
+            VectorSerializer.stream_serialize(CTxIn, self.vin, f)
+            VectorSerializer.stream_serialize(CTxOut, self.vout, f)
+            self.wit.stream_serialize(f)
+        else:
+            VectorSerializer.stream_serialize(CTxIn, self.vin, f)
+            VectorSerializer.stream_serialize(CTxOut, self.vout, f)
         f.write(struct.pack(b"<I", self.nLockTime))
 
     def is_coinbase(self):
         return len(self.vin) == 1 and self.vin[0].prevout.is_null()
 
+    def has_witness(self):
+        """True if witness"""
+        return not self.wit.is_null()
+
     def __repr__(self):
-        return "CTransaction(%r, %r, %i, %i)" % (self.vin, self.vout, self.nLockTime, self.nVersion)
+        return "CTransaction(%r, %r, %i, %i, %r)" % (self.vin, self.vout,
+                self.nLockTime, self.nVersion, self.wit)
 
     @classmethod
     def from_tx(cls, tx):
@@ -354,15 +431,25 @@ class CTransaction(ImmutableSerializable):
             return tx
 
         else:
-            return cls(tx.vin, tx.vout, tx.nLockTime, tx.nVersion)
+            return cls(tx.vin, tx.vout, tx.nLockTime, tx.nVersion, tx.wit)
 
+    def GetTxid(self):
+        """Get the transaction ID.  This differs from the transactions hash as
+            given by GetHash.  GetTxid excludes witness data, while GetHash
+            includes it. """
+        if self.wit != CTxWitness():
+            txid = Hash(CTransaction(self.vin, self.vout, self.nLockTime,
+                self.nVersion).serialize())
+        else:
+            txid = Hash(self.serialize())
+        return txid
 
 @__make_mutable
 class CMutableTransaction(CTransaction):
     """A mutable transaction"""
     __slots__ = []
 
-    def __init__(self, vin=None, vout=None, nLockTime=0, nVersion=1):
+    def __init__(self, vin=None, vout=None, nLockTime=0, nVersion=1, witness=None):
         if not (0 <= nLockTime <= 0xffffffff):
             raise ValueError('CTransaction: nLockTime must be in range 0x0 to 0xffffffff; got %x' % nLockTime)
         self.nLockTime = nLockTime
@@ -376,15 +463,17 @@ class CMutableTransaction(CTransaction):
         self.vout = vout
         self.nVersion = nVersion
 
+        if witness is None:
+            witness = CTxWitness([CTxInWitness() for dummy in range(len(vin))])
+        self.wit = witness
+
     @classmethod
     def from_tx(cls, tx):
         """Create a fully mutable copy of a pre-existing transaction"""
         vin = [CMutableTxIn.from_txin(txin) for txin in tx.vin]
         vout = [CMutableTxOut.from_txout(txout) for txout in tx.vout]
 
-        return cls(vin, vout, tx.nLockTime, tx.nVersion)
-
-
+        return cls(vin, vout, tx.nLockTime, tx.nVersion, tx.wit)
 
 
 class CBlockHeader(ImmutableSerializable):
@@ -440,9 +529,12 @@ class CBlockHeader(ImmutableSerializable):
                 (self.__class__.__name__, self.nVersion, b2lx(self.hashPrevBlock), b2lx(self.hashMerkleRoot),
                  self.nTime, self.nBits, self.nNonce)
 
+class NoWitnessData(Exception):
+    """The block does not have witness data"""
+
 class CBlock(CBlockHeader):
     """A block including all transactions in it"""
-    __slots__ = ['vtx', 'vMerkleTree']
+    __slots__ = ['vtx', 'vMerkleTree', 'vWitnessMerkleTree']
 
     @staticmethod
     def build_merkle_tree_from_txids(txids):
@@ -478,7 +570,7 @@ class CBlock(CBlockHeader):
     @staticmethod
     def build_merkle_tree_from_txs(txs):
         """Build a full merkle tree from transactions"""
-        txids = [tx.GetHash() for tx in txs]
+        txids = [tx.GetTxid() for tx in txs]
         return CBlock.build_merkle_tree_from_txids(txids)
 
     def calc_merkle_root(self):
@@ -491,12 +583,63 @@ class CBlock(CBlockHeader):
             raise ValueError('Block contains no transactions')
         return self.build_merkle_tree_from_txs(self.vtx)[-1]
 
+    @staticmethod
+    def build_witness_merkle_tree_from_txs(txs):
+        """Calculate the witness merkle tree from transactions"""
+        has_witness = False
+        hashes = []
+        for tx in txs:
+            hashes.append(tx.GetHash())
+            has_witness |= tx.has_witness()
+        if not has_witness:
+            raise NoWitnessData
+        hashes[0] = b'\x00' * 32
+        return CBlock.build_merkle_tree_from_txids(hashes)
+
+    def calc_witness_merkle_root(self):
+        """Calculate the witness merkle root
+
+        The calculated merkle root is not cached; every invocation
+        re-calculates it from scratch.
+        """
+        if not len(self.vtx):
+            raise ValueError('Block contains no transactions')
+        return self.build_witness_merkle_tree_from_txs(self.vtx)[-1]
+
+    def get_witness_commitment_index(self):
+        """Find txout # of witness commitment in coinbase
+
+        Return None or an index
+        """
+        if not len(self.vtx):
+            raise ValueError('Block contains no transactions')
+        commit_pos = None
+        for index, out in enumerate(self.vtx[0].vout):
+            script = out.scriptPubKey
+            if len(script) >= 38 and script[:6] == WITNESS_COINBASE_SCRIPTPUBKEY_MAGIC:
+                commit_pos = index
+        if commit_pos is None:
+            raise ValueError('The witness commitment is missed')
+        return commit_pos
+
     def __init__(self, nVersion=2, hashPrevBlock=b'\x00'*32, hashMerkleRoot=b'\x00'*32, nTime=0, nBits=0, nNonce=0, vtx=()):
         """Create a new block"""
+        if vtx:
+            vMerkleTree = tuple(CBlock.build_merkle_tree_from_txs(vtx))
+            if hashMerkleRoot == b'\x00'*32:
+                hashMerkleRoot = vMerkleTree[-1]
+            elif hashMerkleRoot != vMerkleTree[-1]:
+                raise CheckBlockError("CBlock : hashMerkleRoot is not compatible with vtx")
+        else:
+            vMerkleTree = ()
         super(CBlock, self).__init__(nVersion, hashPrevBlock, hashMerkleRoot, nTime, nBits, nNonce)
 
-        vMerkleTree = tuple(CBlock.build_merkle_tree_from_txs(vtx))
         object.__setattr__(self, 'vMerkleTree', vMerkleTree)
+        try:
+            vWitnessMerkleTree = tuple(CBlock.build_witness_merkle_tree_from_txs(vtx))
+        except NoWitnessData:
+            vWitnessMerkleTree = ()
+        object.__setattr__(self, 'vWitnessMerkleTree', vWitnessMerkleTree)
         object.__setattr__(self, 'vtx', tuple(CTransaction.from_tx(tx) for tx in vtx))
 
     @classmethod
@@ -506,13 +649,18 @@ class CBlock(CBlockHeader):
         vtx = VectorSerializer.stream_deserialize(CTransaction, f)
         vMerkleTree = tuple(CBlock.build_merkle_tree_from_txs(vtx))
         object.__setattr__(self, 'vMerkleTree', vMerkleTree)
+        try:
+            vWitnessMerkleTree = tuple(CBlock.build_witness_merkle_tree_from_txs(vtx))
+        except NoWitnessData:
+            vWitnessMerkleTree = ()
+        object.__setattr__(self, 'vWitnessMerkleTree', vWitnessMerkleTree)
         object.__setattr__(self, 'vtx', tuple(vtx))
 
         return self
 
-    def stream_serialize(self, f):
+    def stream_serialize(self, f, include_witness=True):
         super(CBlock, self).stream_serialize(f)
-        VectorSerializer.stream_serialize(CTransaction, self.vtx, f)
+        VectorSerializer.stream_serialize(CTransaction, self.vtx, f, dict(include_witness=include_witness))
 
     def get_header(self):
         """Return the block header
@@ -539,6 +687,10 @@ class CBlock(CBlockHeader):
             object.__setattr__(self, '_cached_GetHash', _cached_GetHash)
             return _cached_GetHash
 
+    def GetWeight(self):
+        """Return the block weight: (stripped_size * 3) + total_size"""
+        return len(self.serialize(dict(include_witness=False))) * 3 + len(self.serialize())
+
 class CoreChainParams(object):
     """Define consensus-critical parameters of a given instance of the Bitcoin system"""
     MAX_MONEY = None
@@ -558,6 +710,10 @@ class CoreTestNetParams(CoreMainParams):
     NAME = 'testnet'
     GENESIS_BLOCK = CBlock.deserialize(x('0100000000000000000000000000000000000000000000000000000000000000000000003ba3edfd7a7b12b27ac72c3e67768f617fc81bc3888a51323a9fb8aa4b1e5e4adae5494dffff001d1aa4ae180101000000010000000000000000000000000000000000000000000000000000000000000000ffffffff4d04ffff001d0104455468652054696d65732030332f4a616e2f32303039204368616e63656c6c6f72206f6e206272696e6b206f66207365636f6e64206261696c6f757420666f722062616e6b73ffffffff0100f2052a01000000434104678afdb0fe5548271967f1a67130b7105cd6a828e03909a67962e0ea1f61deb649f6bc3f4cef38c4f35504e51ec112de5c384df7ba0b8d578a4c702b6bf11d5fac00000000'))
 
+class CoreSigNetParams(CoreMainParams):
+    NAME = 'signet'
+    GENESIS_BLOCK = CBlock.deserialize(x('0100000000000000000000000000000000000000000000000000000000000000000000003ba3edfd7a7b12b27ac72c3e67768f617fc81bc3888a51323a9fb8aa4b1e5e4a008f4d5fae77031e8ad222030101000000010000000000000000000000000000000000000000000000000000000000000000ffffffff4d04ffff001d0104455468652054696d65732030332f4a616e2f32303039204368616e63656c6c6f72206f6e206272696e6b206f66207365636f6e64206261696c6f757420666f722062616e6b73ffffffff0100f2052a01000000434104678afdb0fe5548271967f1a67130b7105cd6a828e03909a67962e0ea1f61deb649f6bc3f4cef38c4f35504e51ec112de5c384df7ba0b8d578a4c702b6bf11d5fac00000000'))
+
 class CoreRegTestParams(CoreTestNetParams):
     NAME = 'regtest'
     GENESIS_BLOCK = CBlock.deserialize(x('0100000000000000000000000000000000000000000000000000000000000000000000003ba3edfd7a7b12b27ac72c3e67768f617fc81bc3888a51323a9fb8aa4b1e5e4adae5494dffff7f20020000000101000000010000000000000000000000000000000000000000000000000000000000000000ffffffff4d04ffff001d0104455468652054696d65732030332f4a616e2f32303039204368616e63656c6c6f72206f6e206272696e6b206f66207365636f6e64206261696c6f757420666f722062616e6b73ffffffff0100f2052a01000000434104678afdb0fe5548271967f1a67130b7105cd6a828e03909a67962e0ea1f61deb649f6bc3f4cef38c4f35504e51ec112de5c384df7ba0b8d578a4c702b6bf11d5fac00000000'))
@@ -580,6 +736,8 @@ def _SelectCoreParams(name):
         coreparams = CoreTestNetParams()
     elif name == 'regtest':
         coreparams = CoreRegTestParams()
+    elif name == 'signet':
+        coreparams = CoreSigNetParams()
     else:
         raise ValueError('Unknown chain %r' % name)
 
@@ -600,7 +758,8 @@ def CheckTransaction(tx):
         raise CheckTransactionError("CheckTransaction() : vout empty")
 
     # Size limits
-    if len(tx.serialize()) > MAX_BLOCK_SIZE:
+    base_tx = CTransaction(tx.vin, tx.vout, tx.nLockTime, tx.nVersion)
+    if len(base_tx.serialize()) > MAX_BLOCK_SIZE:
         raise CheckTransactionError("CheckTransaction() : size limits failed")
 
     # Check for negative or overflow output values
@@ -699,7 +858,8 @@ def CheckBlock(block, fCheckPoW = True, fCheckMerkleRoot = True, cur_time=None):
 
     fCheckPoW        - Check proof-of-work.
 
-    fCheckMerkleRoot - Check merkle root matches transactions.
+    fCheckMerkleRoot - Check merkle root and witness merkle root matches transactions.
+                     - Check witness commitment in coinbase
 
     cur_time         - Current time. Defaults to time.time()
     """
@@ -710,9 +870,12 @@ def CheckBlock(block, fCheckPoW = True, fCheckMerkleRoot = True, cur_time=None):
     # Size limits
     if not block.vtx:
         raise CheckBlockError("CheckBlock() : vtx empty")
-    if len(block.serialize()) > MAX_BLOCK_SIZE:
+    if len(block.serialize(dict(include_witness=False))) > MAX_BLOCK_SIZE:
         raise CheckBlockError("CheckBlock() : block larger than MAX_BLOCK_SIZE")
 
+    if block.GetWeight() > MAX_BLOCK_WEIGHT:
+        raise CheckBlockError("CheckBlock() : block larger than MAX_BLOCK_WEIGHT")
+
     # First transaction must be coinbase
     if not block.vtx[0].is_coinbase():
         raise CheckBlockError("CheckBlock() : first tx is not coinbase")
@@ -730,7 +893,7 @@ def CheckBlock(block, fCheckPoW = True, fCheckMerkleRoot = True, cur_time=None):
 
         CheckTransaction(tx)
 
-        txid = tx.GetHash()
+        txid = tx.GetTxid()
         if txid in unique_txids:
             raise CheckBlockError("CheckBlock() : duplicate transaction")
         unique_txids.add(txid)
@@ -740,8 +903,29 @@ def CheckBlock(block, fCheckPoW = True, fCheckMerkleRoot = True, cur_time=None):
             raise CheckBlockError("CheckBlock() : out-of-bounds SigOpCount")
 
     # Check merkle root
-    if fCheckMerkleRoot and block.hashMerkleRoot != block.calc_merkle_root():
-        raise CheckBlockError("CheckBlock() : hashMerkleRoot mismatch")
+    if fCheckMerkleRoot:
+        if block.hashMerkleRoot != block.calc_merkle_root():
+            raise CheckBlockError("CheckBlock() : hashMerkleRoot mismatch")
+        if len(block.vWitnessMerkleTree):
+            # At least 1 tx has witness: check witness merkle tree
+            root = block.vWitnessMerkleTree[-1]
+            # vtx[0]: coinbase
+            # vtxinwit[0]: first input
+            nonce_script = block.vtx[0].wit.vtxinwit[0].scriptWitness
+            nonce = nonce_script.stack[0]
+            if len(nonce_script.stack) != 1 or len(nonce) != 32:
+                raise CheckBlockError("CheckBlock() : invalid coinbase witnessScript")
+            try:
+                index = block.get_witness_commitment_index()
+            except ValueError as e:
+                raise CheckBlockError("CheckBlock() : " + str(e))
+            commit_script = block.vtx[0].vout[index].scriptPubKey
+            if not (6 + 32 <= len(commit_script) <= 6 + 32 + 1):
+                raise CheckBlockError("CheckBlock() : invalid segwit commitment length")
+            commitment = commit_script[6:6 + 32]
+            commit = commit_script[6:6 + 32]
+            if commit != Hash(root + nonce):
+                raise CheckBlockError("CheckBlock() : invalid segwit commitment")
 
 __all__ = (
         'Hash',
@@ -764,6 +948,8 @@ __all__ = (
         'CMutableTxOut',
         'CTransaction',
         'CMutableTransaction',
+        'CTxWitness',
+        'CTxInWitness',
         'CBlockHeader',
         'CBlock',
         'CoreChainParams',
