@@ -15,18 +15,8 @@ Functionality to build scripts, as well as SignatureHash(). Script evaluation
 is in bitcoin.core.scripteval
 """
 
-from __future__ import absolute_import, division, print_function
 
-import sys
-_bchr = chr
-_bord = ord
-if sys.version > '3':
-    long = int
-    _bchr = lambda x: bytes([x])
-    _bord = lambda x: x
-    from io import BytesIO as _BytesIO
-else:
-    from cStringIO import StringIO as _BytesIO
+from io import BytesIO
 
 import struct
 
@@ -50,9 +40,9 @@ class CScriptOp(int):
     def encode_op_pushdata(d):
         """Encode a PUSHDATA op, returning bytes"""
         if len(d) < 0x4c:
-            return b'' + _bchr(len(d)) + d # OP_PUSHDATA
+            return bytes([len(d)]) + d # OP_PUSHDATA
         elif len(d) <= 0xff:
-            return b'\x4c' + _bchr(len(d)) + d # OP_PUSHDATA1
+            return b'\x4c' + bytes([len(d)]) + d # OP_PUSHDATA1
         elif len(d) <= 0xffff:
             return b'\x4d' + struct.pack(b'<H', len(d)) + d # OP_PUSHDATA2
         elif len(d) <= 0xffffffff:
@@ -235,6 +225,7 @@ OP_NOP1 = CScriptOp(0xb0)
 OP_NOP2 = CScriptOp(0xb1)
 OP_CHECKLOCKTIMEVERIFY = OP_NOP2
 OP_NOP3 = CScriptOp(0xb2)
+OP_CHECKSEQUENCEVERIFY = OP_NOP3
 OP_NOP4 = CScriptOp(0xb3)
 OP_NOP5 = CScriptOp(0xb4)
 OP_NOP6 = CScriptOp(0xb5)
@@ -357,6 +348,7 @@ OPCODE_NAMES.update({
     OP_NOP2: 'OP_NOP2',
     OP_CHECKLOCKTIMEVERIFY: 'OP_CHECKLOCKTIMEVERIFY',
     OP_NOP3: 'OP_NOP3',
+    OP_CHECKSEQUENCEVERIFY: 'OP_CHECKSEQUENCEVERIFY',
     OP_NOP4: 'OP_NOP4',
     OP_NOP5: 'OP_NOP5',
     OP_NOP6: 'OP_NOP6',
@@ -477,6 +469,7 @@ OPCODES_BY_NAME = {
     'OP_NOP2': OP_NOP2,
     'OP_CHECKLOCKTIMEVERIFY': OP_CHECKLOCKTIMEVERIFY,
     'OP_NOP3': OP_NOP3,
+    'OP_CHECKSEQUENCEVERIFY': OP_CHECKSEQUENCEVERIFY,
     'OP_NOP4': OP_NOP4,
     'OP_NOP5': OP_NOP5,
     'OP_NOP6': OP_NOP6,
@@ -521,12 +514,12 @@ class CScript(bytes):
     def __coerce_instance(cls, other):
         # Coerce other into bytes
         if isinstance(other, CScriptOp):
-            other = _bchr(other)
-        elif isinstance(other, (int, long)):
+            other = bytes([other])
+        elif isinstance(other, int):
             if 0 <= other <= 16:
-                other = bytes(_bchr(CScriptOp.encode_op_n(other)))
+                other = bytes([CScriptOp.encode_op_n(other)])
             elif other == -1:
-                other = bytes(_bchr(OP_1NEGATE))
+                other = bytes([OP_1NEGATE])
             else:
                 other = CScriptOp.encode_op_pushdata(bitcoin.core._bignum.bn2vch(other))
         elif isinstance(other, (bytes, bytearray)):
@@ -569,7 +562,7 @@ class CScript(bytes):
         i = 0
         while i < len(self):
             sop_idx = i
-            opcode = _bord(self[i])
+            opcode = self[i]
             i += 1
 
             if opcode > OP_PUSHDATA4:
@@ -585,21 +578,21 @@ class CScript(bytes):
                     pushdata_type = 'PUSHDATA1'
                     if i >= len(self):
                         raise CScriptInvalidError('PUSHDATA1: missing data length')
-                    datasize = _bord(self[i])
+                    datasize = self[i]
                     i += 1
 
                 elif opcode == OP_PUSHDATA2:
                     pushdata_type = 'PUSHDATA2'
                     if i + 1 >= len(self):
                         raise CScriptInvalidError('PUSHDATA2: missing data length')
-                    datasize = _bord(self[i]) + (_bord(self[i+1]) << 8)
+                    datasize = self[i] + (self[i+1] << 8)
                     i += 2
 
                 elif opcode == OP_PUSHDATA4:
                     pushdata_type = 'PUSHDATA4'
                     if i + 3 >= len(self):
                         raise CScriptInvalidError('PUSHDATA4: missing data length')
-                    datasize = _bord(self[i]) + (_bord(self[i+1]) << 8) + (_bord(self[i+2]) << 16) + (_bord(self[i+3]) << 24)
+                    datasize = self[i] + (self[i+1] << 8) + (self[i+2] << 16) + (self[i+3] << 24)
                     i += 4
 
                 else:
@@ -673,14 +666,28 @@ class CScript(bytes):
         Note that this test is consensus-critical.
         """
         return (len(self) == 23 and
-                _bord(self[0]) == OP_HASH160 and
-                _bord(self[1]) == 0x14 and
-                _bord(self[22]) == OP_EQUAL)
+                self[0] == OP_HASH160 and
+                self[1] == 0x14 and
+                self[22] == OP_EQUAL)
 
     def is_witness_scriptpubkey(self):
-        """Returns true if this is a scriptpubkey signaling segregated witness
-        data. """
-        return 3 <= len(self) <= 42 and CScriptOp(struct.unpack('<b',self[0:1])[0]).is_small_int()
+        """Returns true if this is a scriptpubkey signaling segregated witness data.
+
+        A witness program is any valid CScript that consists of a 1-byte push opcode
+        followed by a data push between 2 and 40 bytes.
+        """
+        size = len(self)
+        if size < 4 or size > 42:
+            return False
+
+        head = struct.unpack('<bb', self[:2])
+        if not CScriptOp(head[0]).is_small_int():
+            return False
+
+        if head[1] + 2 != size:
+            return False
+
+        return True
 
     def witness_version(self):
         """Returns the witness version on [0,16]. """
@@ -691,7 +698,7 @@ class CScript(bytes):
         return len(self) == 22 and self[0:2] == b'\x00\x14'
 
     def is_witness_v0_nested_keyhash(self):
-        """Returns true if this is a scriptpubkey for V0 P2WPKH embedded in P2SH. """
+        """Returns true if this is a scriptSig for V0 P2WPKH embedded in P2SH. """
         return len(self) == 23 and self[0:3] == b'\x16\x00\x14'
 
     def is_witness_v0_scripthash(self):
@@ -699,8 +706,8 @@ class CScript(bytes):
         return len(self) == 34 and self[0:2] == b'\x00\x20'
 
     def is_witness_v0_nested_scripthash(self):
-        """Returns true if this is a scriptpubkey for V0 P2WSH embedded in P2SH. """
-        return len(self) == 23 and self[0:2] == b'\xa9\x14' and self[-1] == b'\x87'
+        """Returns true if this is a scriptSig for V0 P2WSH embedded in P2SH. """
+        return len(self) == 35 and self[0:3] == b'\x22\x00\x20'
 
     def is_push_only(self):
         """Test if the script only contains pushdata ops
@@ -730,7 +737,7 @@ class CScript(bytes):
                 if op > OP_16:
                     continue
 
-                elif op < OP_PUSHDATA1 and op > OP_0 and len(data) == 1 and _bord(data[0]) <= 16:
+                elif op < OP_PUSHDATA1 and op > OP_0 and len(data) == 1 and data[0] <= 16:
                     # Could have used an OP_n code, rather than a 1-byte push.
                     return False
 
@@ -753,7 +760,7 @@ class CScript(bytes):
     def is_unspendable(self):
         """Test if the script is provably unspendable"""
         return (len(self) > 0 and
-                _bord(self[0]) == OP_RETURN)
+                self[0] == OP_RETURN)
 
     def is_valid(self):
         """Return True if the script is valid, False otherwise
@@ -1001,7 +1008,7 @@ def SignatureHash(script, txTo, inIdx, hashtype, amount=None, sigversion=SIGVERS
             serialize_outputs = txTo.vout[inIdx].serialize()
             hashOutputs = bitcoin.core.Hash(serialize_outputs)
 
-        f = _BytesIO()
+        f = BytesIO()
         f.write(struct.pack("<i", txTo.nVersion))
         f.write(hashPrevouts)
         f.write(hashSequence)
@@ -1138,6 +1145,7 @@ __all__ = (
         'OP_NOP2',
         'OP_CHECKLOCKTIMEVERIFY',
         'OP_NOP3',
+        'OP_CHECKSEQUENCEVERIFY',
         'OP_NOP4',
         'OP_NOP5',
         'OP_NOP6',
