@@ -9,7 +9,6 @@
 # propagated, or distributed except according to the terms contained in the
 # LICENSE file.
 
-from __future__ import absolute_import, division, print_function, unicode_literals
 
 import hashlib
 import random
@@ -19,14 +18,7 @@ import time
 # Py3 compatibility
 import sys
 
-if sys.version > '3':
-    _bchr = lambda x: bytes([x])
-    _bord = lambda x: x[0]
-    from io import BytesIO as _BytesIO
-else:
-    _bchr = chr
-    _bord = ord
-    from cStringIO import StringIO as _BytesIO
+from io import BytesIO
 
 # Bad practice, so we have a __all__ at the end; this should be cleaned up
 # later.
@@ -59,7 +51,7 @@ class MsgSerializable(Serializable):
         raise NotImplementedError
 
     def to_bytes(self):
-        f = _BytesIO()
+        f = BytesIO()
         self.msg_ser(f)
         body = f.getvalue()
         res = bitcoin.params.MESSAGE_START
@@ -77,7 +69,7 @@ class MsgSerializable(Serializable):
 
     @classmethod
     def from_bytes(cls, b, protover=PROTO_VERSION):
-        f = _BytesIO(b)
+        f = BytesIO(b)
         return MsgSerializable.stream_deserialize(f, protover=protover)
 
     @classmethod
@@ -107,7 +99,7 @@ class MsgSerializable(Serializable):
         if command in messagemap:
             cls = messagemap[command]
             #        print("Going to deserialize '%s'" % msg)
-            return cls.msg_deser(_BytesIO(msg))
+            return cls.msg_deser(BytesIO(msg))
         else:
             print("Command '%s' not in messagemap" % repr(command))
             return None
@@ -131,6 +123,7 @@ class msg_version(MsgSerializable):
         self.strSubVer = (b'/python-bitcoinlib:' +
                           bitcoin.__version__.encode('ascii') + b'/')
         self.nStartingHeight = -1
+        self.fRelay = True
 
     @classmethod
     def msg_deser(cls, f, protover=PROTO_VERSION):
@@ -154,8 +147,12 @@ class msg_version(MsgSerializable):
             c.nNonce = None
             c.strSubVer = None
             c.nStartingHeight = None
+        if c.nVersion >= 70001:
+            c.fRelay = struct.unpack(b"<B", ser_read(f,1))[0]
+        else:
+            c.fRelay = True
         return c
- 
+
     def msg_ser(self, f):
         f.write(struct.pack(b"<i", self.nVersion))
         f.write(struct.pack(b"<Q", self.nServices))
@@ -165,9 +162,10 @@ class msg_version(MsgSerializable):
         f.write(struct.pack(b"<Q", self.nNonce))
         VarStringSerializer.stream_serialize(self.strSubVer, f)
         f.write(struct.pack(b"<i", self.nStartingHeight))
+        f.write(struct.pack(b"<B", self.fRelay))
 
     def __repr__(self):
-        return "msg_version(nVersion=%i nServices=%i nTime=%s addrTo=%s addrFrom=%s nNonce=0x%016X strSubVer=%s nStartingHeight=%i)" % (self.nVersion, self.nServices, time.ctime(self.nTime), repr(self.addrTo), repr(self.addrFrom), self.nNonce, self.strSubVer, self.nStartingHeight)
+        return "msg_version(nVersion=%i nServices=%i nTime=%s addrTo=%s addrFrom=%s nNonce=0x%016X strSubVer=%s nStartingHeight=%i fRelay=%r)" % (self.nVersion, self.nServices, time.ctime(self.nTime), repr(self.addrTo), repr(self.addrFrom), self.nNonce, self.strSubVer, self.nStartingHeight, self.fRelay)
 
 
 class msg_verack(MsgSerializable):
