@@ -23,31 +23,14 @@ a different implementation can be used instead, at your own risk:
 thus better optimized but perhaps less stable.)
 """
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-import ssl
-
-try:
-    import http.client as httplib
-except ImportError:
-    import httplib
+import http.client as httplib
 import base64
 import binascii
 import decimal
 import json
 import os
 import platform
-import sys
-try:
-    import urllib.parse as urlparse
-except ImportError:
-    import urlparse
-
-# needed for python2 compatibility
-if sys.version < '3':
-    try:
-        FileNotFoundError
-    except NameError:
-        FileNotFoundError = IOError
+import urllib.parse as urlparse
 
 import bitcoin
 from bitcoin.core import COIN, x, lx, b2lx, CBlock, CBlockHeader, CTransaction, COutPoint, CTxOut
@@ -58,12 +41,35 @@ DEFAULT_USER_AGENT = "AuthServiceProxy/0.1"
 
 DEFAULT_HTTP_TIMEOUT = 30
 
-# (un)hexlify to/from unicode, needed for Python3
-unhexlify = binascii.unhexlify
-hexlify = binascii.hexlify
-if sys.version > '3':
-    unhexlify = lambda h: binascii.unhexlify(h.encode('utf8'))
-    hexlify = lambda b: binascii.hexlify(b).decode('utf8')
+def unhexlify_str(h):
+    """
+    Converts a string containing hexadecimal encoding into a bytes-object.
+
+    It works by encoding the given string h as ASCII, then interpreting each of
+    its two-character ASCII tuples as bytes:
+        - "00" to byte 0
+        - "ff" to byte 255
+        - "FF" also to byte 255
+
+    The string must only contain characters in the ranges 0-9, a-f and A-F.
+
+    If the string contains characters not in ASCII,
+        UnicodeEncodeError is raised.
+    If the string contains out-of-range ASCII characters,
+        binascii.Error is raised.
+    If number of encoded ASCII bytes is odd,
+        binascii.Error is raised.
+    """
+    return binascii.unhexlify(h.encode('ascii'))
+
+def hexlify_str(b):
+    """
+    Given an arbitrary bytes-like object, returns a string (that would encode
+    as ASCII) containing the hex-representation of the bytes-like object.
+
+    Always succeeds.
+    """
+    return binascii.hexlify(b).decode('ascii')
 
 
 class JSONRPCError(Exception):
@@ -130,7 +136,8 @@ class BaseProxy(object):
                  service_url=None,
                  service_port=None,
                  btc_conf_file=None,
-                 timeout=DEFAULT_HTTP_TIMEOUT):
+                 timeout=DEFAULT_HTTP_TIMEOUT,
+                 connection=None):
 
         # Create a dummy connection early on so if __init__() fails prior to
         # __conn being created __del__() can detect the condition and handle it
@@ -211,8 +218,11 @@ class BaseProxy(object):
             authpair = authpair.encode('utf8')
             self.__auth_header = b"Basic " + base64.b64encode(authpair)
 
-        self.__conn = httplib.HTTPConnection(self.__url.hostname, port=port,
-                                             timeout=timeout)
+        if connection:
+            self.__conn = connection
+        else:
+            self.__conn = httplib.HTTPConnection(self.__url.hostname, port=port,
+                                                 timeout=timeout)
 
     def _call(self, service_name, *args):
         self.__id_count += 1
@@ -234,8 +244,13 @@ class BaseProxy(object):
         self.__conn.request('POST', self.__url.path, postdata, headers)
 
         response = self._get_response()
-        if response['error'] is not None:
-            raise JSONRPCError(response['error'])
+        err = response.get('error')
+        if err is not None:
+            if isinstance(err, dict):
+                raise JSONRPCError(
+                    {'code': err.get('code', -345),
+                     'message': err.get('message', 'error message not specified')})
+            raise JSONRPCError({'code': -344, 'message': str(err)})
         elif 'result' not in response:
             raise JSONRPCError({
                 'code': -343, 'message': 'missing JSON-RPC result'})
@@ -263,8 +278,15 @@ class BaseProxy(object):
             raise JSONRPCError({
                 'code': -342, 'message': 'missing HTTP response from server'})
 
-        return json.loads(http_response.read().decode('utf8'),
-                          parse_float=decimal.Decimal)
+        rdata = http_response.read().decode('utf8')
+        try:
+            return json.loads(rdata, parse_float=decimal.Decimal)
+        except Exception:
+            raise JSONRPCError({
+                'code': -342,
+                'message': ('non-JSON HTTP response with \'%i %s\' from server: \'%.20s%s\''
+                            % (http_response.status, http_response.reason,
+                               rdata, '...' if len(rdata) > 20 else ''))})
 
     def close(self):
         if self.__conn is not None:
@@ -333,7 +355,7 @@ class Proxy(BaseProxy):
         out of the file ``btc_conf_file``. If ``btc_conf_file`` is not
         specified, ``~/.bitcoin/bitcoin.conf`` or equivalent is used by
         default.  The default port is set according to the chain parameters in
-        use: mainnet, testnet, or regtest.
+        use: mainnet, testnet, signet, or regtest.
 
         Usually no arguments to ``Proxy()`` are needed; the local bitcoind will
         be used.
@@ -370,10 +392,10 @@ class Proxy(BaseProxy):
          'changepos': Position of added change output, or -1,
         }
         """
-        hextx = hexlify(tx.serialize())
+        hextx = hexlify_str(tx.serialize())
         r = self._call('fundrawtransaction', hextx, include_watching)
 
-        r['tx'] = CTransaction.deserialize(unhexlify(r['hex']))
+        r['tx'] = CTransaction.deserialize(unhexlify_str(r['hex']))
         del r['hex']
 
         r['fee'] = int(r['fee'] * COIN)
@@ -383,7 +405,7 @@ class Proxy(BaseProxy):
     def generate(self, numblocks):
         """
         DEPRECATED (will be removed in bitcoin-core v0.19)
-        
+
         Mine blocks immediately (before the RPC call returns)
 
         numblocks - How many blocks are generated immediately.
@@ -392,10 +414,10 @@ class Proxy(BaseProxy):
         """
         r = self._call('generate', numblocks)
         return (lx(blk_hash) for blk_hash in r)
-    
+
     def generatetoaddress(self, numblocks, addr):
         """Mine blocks immediately (before the RPC call returns) and
-        allocate block reward to passed address. Replaces deprecated 
+        allocate block reward to passed address. Replaces deprecated
         "generate(self,numblocks)" method.
 
         numblocks - How many blocks are generated immediately.
@@ -461,7 +483,7 @@ class Proxy(BaseProxy):
                     'nextblockhash':nextblockhash,
                     'chainwork':x(r['chainwork'])}
         else:
-            return CBlockHeader.deserialize(unhexlify(r))
+            return CBlockHeader.deserialize(unhexlify_str(r))
 
 
     def getblock(self, block_hash):
@@ -482,7 +504,7 @@ class Proxy(BaseProxy):
         except InvalidAddressOrKeyError as ex:
             raise IndexError('%s.getblock(): %s (%d)' %
                     (self.__class__.__name__, ex.error['message'], ex.error['code']))
-        return CBlock.deserialize(unhexlify(r))
+        return CBlock.deserialize(unhexlify_str(r))
 
     def getblockcount(self):
         """Return the number of blocks in the longest block chain"""
@@ -544,7 +566,7 @@ class Proxy(BaseProxy):
             r = [lx(txid) for txid in r]
             return r
 
-    def getrawtransaction(self, txid, verbose=False):
+    def getrawtransaction(self, txid, verbose=False, block_hash=None):
         """Return transaction with hash txid
 
         Raises IndexError if transaction not found.
@@ -552,16 +574,22 @@ class Proxy(BaseProxy):
         verbose - If true a dict is returned instead with additional
         information on the transaction.
 
+        block_hash - Hash of the block containing the transaction
+        (required when transaction not currently indexed by Bitcoin Core)
+
         Note that if all txouts are spent and the transaction index is not
         enabled the transaction may not be available.
         """
         try:
-            r = self._call('getrawtransaction', b2lx(txid), 1 if verbose else 0)
+            if block_hash is None:
+                r = self._call('getrawtransaction', b2lx(txid), 1 if verbose else 0)
+            else:
+                r = self._call('getrawtransaction', b2lx(txid), 1 if verbose else 0, b2lx(block_hash))
         except InvalidAddressOrKeyError as ex:
             raise IndexError('%s.getrawtransaction(): %s (%d)' %
                     (self.__class__.__name__, ex.error['message'], ex.error['code']))
         if verbose:
-            r['tx'] = CTransaction.deserialize(unhexlify(r['hex']))
+            r['tx'] = CTransaction.deserialize(unhexlify_str(r['hex']))
             del r['hex']
             del r['txid']
             del r['version']
@@ -570,8 +598,7 @@ class Proxy(BaseProxy):
             del r['vout']
             r['blockhash'] = lx(r['blockhash']) if 'blockhash' in r else None
         else:
-            r = CTransaction.deserialize(unhexlify(r))
-
+            r = CTransaction.deserialize(unhexlify_str(r))
         return r
 
     def getreceivedbyaddress(self, addr, minconf=1):
@@ -618,7 +645,7 @@ class Proxy(BaseProxy):
             raise IndexError('%s.gettxout(): unspent txout %r not found' % (self.__class__.__name__, outpoint))
 
         r['txout'] = CTxOut(int(r['value'] * COIN),
-                            CScript(unhexlify(r['scriptPubKey']['hex'])))
+                            CScript(unhexlify_str(r['scriptPubKey']['hex'])))
         del r['value']
         del r['scriptPubKey']
         r['bestblock'] = lx(r['bestblock'])
@@ -658,7 +685,7 @@ class Proxy(BaseProxy):
                 unspent['address'] = CBitcoinAddress(unspent['address'])
             except KeyError:
                 pass
-            unspent['scriptPubKey'] = CScript(unhexlify(unspent['scriptPubKey']))
+            unspent['scriptPubKey'] = CScript(unhexlify_str(unspent['scriptPubKey']))
             unspent['amount'] = int(unspent['amount'] * COIN)
             r2.append(unspent)
         return r2
@@ -674,7 +701,7 @@ class Proxy(BaseProxy):
 
         allowhighfees - Allow even if fees are unreasonably high.
         """
-        hextx = hexlify(tx.serialize())
+        hextx = hexlify_str(tx.serialize())
         r = None
         if allowhighfees:
             r = self._call('sendrawtransaction', hextx, True)
@@ -704,9 +731,21 @@ class Proxy(BaseProxy):
 
         FIXME: implement options
         """
-        hextx = hexlify(tx.serialize())
+        hextx = hexlify_str(tx.serialize())
         r = self._call('signrawtransaction', hextx, *args)
-        r['tx'] = CTransaction.deserialize(unhexlify(r['hex']))
+        r['tx'] = CTransaction.deserialize(unhexlify_str(r['hex']))
+        del r['hex']
+        return r
+
+    def signrawtransactionwithwallet(self, tx, *args):
+        """Sign inputs for transaction
+            bicoincore >= 0.17.x
+
+        FIXME: implement options
+        """
+        hextx = hexlify_str(tx.serialize())
+        r = self._call('signrawtransactionwithwallet', hextx, *args)
+        r['tx'] = CTransaction.deserialize(unhexlify_str(r['hex']))
         del r['hex']
         return r
 
@@ -716,7 +755,7 @@ class Proxy(BaseProxy):
         params is optional and is currently ignored by bitcoind. See
         https://en.bitcoin.it/wiki/BIP_0022 for full specification.
         """
-        hexblock = hexlify(block.serialize())
+        hexblock = hexlify_str(block.serialize())
         if params is not None:
             return self._call('submitblock', hexblock, params)
         else:
@@ -728,7 +767,7 @@ class Proxy(BaseProxy):
         if r['isvalid']:
             r['address'] = CBitcoinAddress(r['address'])
         if 'pubkey' in r:
-            r['pubkey'] = unhexlify(r['pubkey'])
+            r['pubkey'] = unhexlify_str(r['pubkey'])
         return r
 
     def unlockwallet(self, password, timeout=60):
